package org.jabber.applet;
import java.awt.Color;
import java.util.Enumeration;
import java.util.Hashtable;
import org.jabber.applet.awt.*;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/


/**
 * This class is designed to communicate between the main jabber client antenna and it's 
 * numerous/infinite child chat windows.
 * 
 * @author David Scott
 */
class ChatWindows {
    /**
     * Hash table of users we're chatting
     * Key: username@server
     * Value: chat dialog window
     */
    Hashtable chats = new Hashtable();

    /**
     * Background color for chat windows
     */
    private Color clientColor;

    /* Hash table of JIDs' status
     * key: username@server
     * value: status
     */
    Hashtable jidStatus = new Hashtable();
    /**
     * Holding place for chat window communications
     * 
     * @param clientColor Background color for all newly created child chat windows
     */
    ChatWindows(Color clientColor) {
        this.clientColor = clientColor;
    }

    /**
     * Start a chat window.
     * Create a chat window corresponding to the input user.
     * @param jidMe JID describing current user
     * @param jidYou JID describing user we're going to chat with
     * @param left location of new chat window
     * @param right location of new chat window
     */
    protected void startChatn(String sTitle, JID jidMe, JID jidYou, int left, int top) {
        this.startChatn(sTitle, jidMe, jidYou, left, top, false);
    }
    protected void startChatn(String sTitle, JID jidMe, JID jidYou, int left, int top, boolean bInvite) {
        //------------------------------------------
        // Look for existing chat with selected user.
        // If one is found do nothing.
        // Note: we're currently doing nothing vs popping new chat window to top
        //       this is due to fact when windows screen saver is on and a msg is received
        //       the chat window gets the focus but user doesn't see chat window getting
        //       focus, so he/she may enter screen password (security flaw in windows)
        //------------------------------------------
        if (chats.containsKey(jidYou.getJID()))
            ; // jid found, do nothing
        else {                               // new window needed
            MsgChat msgDialog;          // create chat window
            msgDialog = new MsgChat(clientColor, Config.CLIENT_FONT, jidMe.getNick(), jidYou, true, bInvite);

            // Create chat window
            msgDialog.setSize(300, 220);
            msgDialog.setLocation(left, top);
            msgDialog.setTitle(sTitle);
            msgDialog.show();

            // Place window in chats table
            chats.put(jidYou.getJID(), msgDialog);
        }                               // end, new window needed
    }                                   // end, chatWindowStart routine

    protected void startGroupChatn(String sTitle, String Room, String grpChatServer) {
        MsgChat GroupMsgDialog;
    }

    /**
     * Find the index of the user in the user/chat window vectors
     * @param findJID JID to search for
     * @param jidYou
     * @return JID searched for
     */
    protected boolean find(JID jidYou) {
        if (chats.containsKey(jidYou.getJID()))
            return(true);
        else
            return(false);
    }

    /**
     * Clear Chat Windows and corresponding vectors used to track them.
     */
    protected void clearAll() {
        Enumeration e = chats.keys();
        while (e.hasMoreElements())     // clear all chat window objects
        {
            Object obj = chats.get(e.nextElement());
            ((MsgChat)obj).dispose();
        }
        chats.clear();
    }

    /**
     * Chat Window Closing
     * Cleanup corresponding item in tracking table and child windows
     * @param jidYou user whos chat window is closing
     */
    protected void closing(JID jidYou) {
        // if chat child window exists dispose of it
        Object obj = chats.get(jidYou.getJID());
        if (obj != null)
            ((MsgChat)obj).dispose();

        chats.remove(jidYou.getJID());  // remove chat window tracking element
    }

    /**
     * Send message just received to the corresponding chat window
     * @param jidYou user we're chatn with
     * @param sBody body of message
     */
    protected void msgReceived(JID jidYou, String sBody) {
        Object obj = chats.get(jidYou.getJID());
        if (obj != null)
            ((MsgChat)obj).msgReceived(jidYou, sBody);      // send msg to chat window
    }


    /**
     * System msg received, forward to chat window
     * @param jidYou user we're chatn with
     * @param sStatus new status
     */
    protected void systemMsg(JID jidYou, String sStatus) {
        Object obj = chats.get(jidYou.getJID());
        if (obj != null)
            ((MsgChat)obj).systemMsg(sStatus);   // send status to chat window
    }

    /**
     * A reply to your message has been started 
     * @param jidYou user we're chatn with
     * @param msg message to display
     */
    protected void msgReplyStarted(JID jidYou, String msg) {
        Object obj = chats.get(jidYou.getJID());
        if (obj != null)
            ((MsgChat)obj).msgReplyStarted(jidYou, msg);
    }

    /**
     * Send new status of user to corresponding chat window
     * @param jidYou user we're chatn with
     * @param sStatus new users status
     */
    protected void statusSet(JID jidYou, String sStatus) {
        Object obj = chats.get(jidYou.getJID());
        jidStatus.put(jidYou.getJID(), sStatus);
        if (obj != null) {
            ((MsgChat)obj).changeStatus(jidYou, sStatus);   // send status to chat window
        }
    }

    protected String statusGet(JID jidYou) {
        String sStatus = "";
        if (jidStatus.containsKey(jidYou.getJID())) {
            sStatus = (String)jidStatus.get(jidYou.getJID());
            if (sStatus.length() > 0) {
                return sStatus;
            } else {
                return "";
            }
        } else {
            return "";
        }
    }
}
