package org.jabber.applet;

import java.awt.*;
import java.util.Vector;
import java.util.Enumeration;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ or at http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * This class is used for a Jabber Roster List.
 * The list of users are displayed via a list of groups.
 * Within each group are n users.
 * 
 * @author David Scott
 */
public class JIDList {
    /**
     * Vector of groups in the list
     */
    private Vector vGroups = new Vector();

    /**
     * Vector of objects in the list model.
     */
    private Vector list = new Vector();

    /**
     * Vector of RosterListeners.
     */
    private Vector rosterListeners = new Vector();

    /**
     * index of selected item.
     */
    private int selectedIndex = -1;

    /**
     * JIDGroup Constructor
     */
    public JIDList() {}

    /**
     * Add a roster listener to be notified when the roster changes.
     */
    public void addRosterListener( RosterListener listener ) {
        if( !rosterListeners.contains(listener) )
            rosterListeners.addElement(listener);
    }
    /**
     * Remove a roster listener.
     */
    public void removeRosterListener( RosterListener listener ) {
        rosterListeners.removeElement(listener);
    }
    /**
     * Fire an event to indicate that the roster changed.
     */
    protected void fireRosterChange( ) {
        java.util.EventObject eo = new java.util.EventObject(this);
        Vector v = (Vector)(rosterListeners.clone());
        for( int i= 0; i< v.size(); i++ ) {
            RosterListener rl = ((RosterListener)v.elementAt(i));
            if( rl != null )
                rl.rosterChanged(eo);
        }
    }

    /**
     * Get the groupname that the input jid is a member of
     * @param jid jid to find corresponding jid
     * @param nMask specifying how much detail to use when looking for jid
     * @return group name
     */
    public String getGroupName(JID jid, int nMask) {
        // for each group
        Enumeration e = vGroups.elements();
        JID fndJID;
        while (e.hasMoreElements()) {                                   // loop thru all groups in list
            Object obj = e.nextElement();
            fndJID = ((JIDGroup)obj).getJID(jid, nMask);
            if (fndJID != null)             // found jid in this group?
                return(((JIDGroup)obj).getName());             // return the group name
        }                                   // end, loop thru all groups in list
        return(null);                       // matching jid not found
    }

    /**
     * Create/Display the roster list with all groups expanded
     */
    public void setAllExpanded() {
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {                                       // step thru all groups
            JIDGroup obj = (JIDGroup)e.nextElement();
            (obj).setExpanded();                // set to expanded?
        }                                       // end, step thru all groups
        display();                              // display the new roster
    }

    /**
     * Create/Display the roster list.
     * The list consists of a vector of groups.
     */
    protected void display() {
        Vector newList = new Vector();
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {                                       // step thru all groups
            JIDGroup obj = (JIDGroup)e.nextElement();
            newList.addElement(obj);
            if ((obj).getExpanded())  // group expanded?
            {                                   // add group name to list
                Vector v = obj.getJIDs();
                if( v != null ) {
                    for(int i = 0; i< v.size(); i++ ) {
                        newList.addElement( v.elementAt(i) );
                    }
                }
            }
        }                                       // end, step thru all groups
        this.list = newList;
        fireRosterChange( );
    }

    /**
     * Create XML representation of list contents
     * @param myNick my nickname
     * @return XML string of all roster list contacts
     */
    public String toXML(String myNick) {
        //Use a vector to keep track of who is in the list
        //(to avoid duplicates)
        Vector vNames = new Vector(20,10);
        String XMLItems = "";
        JID oJID;

        //Add your own nick to vector so that it won't get added to list
        //(will think it's already added when check performed below)
        vNames.addElement(myNick);

        //XML Format: <Item><Display>Jonathan Paulson</Display><Value>thejonz@jabber.com</Value></Item>
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {
            // step thru all groups
            JIDGroup obj = (JIDGroup)e.nextElement();
            Vector v = obj.getJIDs();
            if( v != null ) {
                for(int i = 0; i< v.size(); i++ ) {
                    oJID = (JID) v.elementAt(i);
                    //Make sure name isn't already in list and don't add if the name is from the group chat
                    //(contains "@groups." in the JabberID
                    if (vNames.indexOf(oJID.getNick()) == -1 && oJID.getJID().indexOf("@groups.") == -1) {
                        XMLItems +="<Item><Display>" + oJID.getNick() + " (" + oJID.getJID() + ")" + "</Display><Value>" + oJID.getJID() + "</Value></Item>";
                        vNames.addElement(oJID.getNick());
                    }
                }
            }
        }                                       // end, step thru all groups
        return XMLItems;
    }

    /**
     * Update the display status of the input JID user
     * @param jidUpdate JID who's status is being updated
     * @param sStatus New status of user
     * @param sShow sort of presence
     * @return true - successful update, false - failed update
     */
    public boolean jidUpdate(JID jidUpdate, String sStatus, String sShow, int nMask) {
        // 1. find index of user
        // 2. update status
        // 3. if user is currently displayed update display
        if (sShow.equals("")) {
            sShow = "online";
        }
        JID xJID = getJID(jidUpdate, nMask);
        if (xJID != null  ) {
            String oldStatus = xJID.getStatus();
            if( oldStatus != null && !oldStatus.equals(sStatus) ) {
                // actual change in status.
                xJID.setStatus(sStatus, sShow);
                // Search thru list to see if user is displayed
                // If yes then update the display.
                if( list.contains(xJID) ) {
                    display();
                }
            }
            return( true );
        } else
            return( false );
    }

    /**
     * Remove all groups from the list
     */
    public void removeAll() {
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {
            Object obj = e.nextElement();
            ((JIDGroup)obj).removeAll();
        }
        vGroups.removeAllElements();
        display();
    }


    /**
     * Remove all members in a group
     * Needed for groupchat when window closes
     * @param oldGroup name of group being removed
     * @return true - successful remove, false - failed
     */
    public void groupRemove(String oldGroup) {
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {
            Object obj = e.nextElement();
            if (((JIDGroup)obj).getName().equals(oldGroup)) {
                ((JIDGroup)obj).removeAll();
                vGroups.removeElement(obj);
                display();
            }
        }
    }

    /**
     * Remove the specified user from the list
     * @param jid member to remove
     * @param nMask specify detail in which to find jid
     */
    public boolean jidRemove(JID jid, int nMask) {
        // find the group
        // for each group delete member
        boolean bSuccess = false;
        Enumeration e = vGroups.elements();
        int i = 0;
        while ((e.hasMoreElements()) && (bSuccess == false)) {
            Object obj = e.nextElement();
            if (((JIDGroup)obj).jidRemove(jid, nMask) == true) {
                if (((JIDGroup)obj).getJIDCount() == 0)
                    vGroups.removeElementAt(i);
                display();
                return (true);
            }
            i++;
        }
        return( false );
    }



    /**
     * Add a new member to the list.
     * @param grpName Group to add the user into
     * @param newMember New member JID to be added to the specified group
     */
    public void jidAdd(String grpName, JID newMember) {
        jidAdd(grpName,newMember,true);
    }
    public void jidAdd(String grpName, JID newMember, boolean display) {
        long t1 = System.currentTimeMillis();
        if ((newMember == null) || (newMember.getNick() == "")) {
            return;
        }

        // if group doesn't exist, add it
        int i = groupIndex(grpName);
        if (i == -1) {
            JIDGroup newGroup = new JIDGroup(grpName);
            i = groupAdd(newGroup);
        }

        // add member to group
        // loop thru the vector, as soon as find group add the user
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {
            Object obj = e.nextElement();
            String sVector = ((JIDGroup)obj).getName();
            if ( grpName.equalsIgnoreCase(sVector)) {
                ((JIDGroup)obj).jidAdd(newMember);
            }
        }
        if( display )
            display();
        long t2 = System.currentTimeMillis();
    }

    /**
     * Add the specified group to the list
     * @param newGroup Group to be added to the list.
     * @return index of where element was added
     */
    private int groupAdd(JIDGroup newGroup) {
        int ndxSearch = 0;
        int ndxInsert = -1;
        boolean bSlippedIn = false;

        // loop thru the vector
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {
            Object obj = e.nextElement();
            String sVector = ((JIDGroup)obj).getName();
            if( newGroup.getName().toUpperCase().compareTo(sVector.toUpperCase()) < 0 ) {
                if (bSlippedIn == false) {
                    ndxInsert = ndxSearch;
                    bSlippedIn = true;
                }
            }
            ndxSearch++;
        }
        if (bSlippedIn == false)
            ndxInsert = ndxSearch;
        vGroups.insertElementAt(newGroup, ndxInsert);    // save the JID
        return ndxInsert;
    }

    /**
     * Determine if a jid is selected
     * @return true a jid is selected, false a jid is not selected
     */
    public boolean isJIDSelected() {
        Object o = getSelectedItem();
        if( o != null && o instanceof JID )
            return true;
        return false;
    }

    /**
     * Determine if a group is selected
     * @return true a group is selected, false a group is not selected
     */
    public boolean isGroupSelected() {
        Object o = getSelectedItem();
        if( o != null && o instanceof JIDGroup )
            return true;
        return false;
    }

    /**
     * Get the selected item/object
     * @return selected item, null if nothing selected
     */
    public Object getSelectedItem() {
        if( selectedIndex >= 0 && selectedIndex < list.size() )
            return list.elementAt( selectedIndex );
        return null;
    }

    /**
     * Get index of groupname
     * @param sGroupName string of group to find
     * @return index of group
     */
    public int groupIndex(String sGroupName) {
        int i = -1;
        Enumeration e = vGroups.elements();
        while (e.hasMoreElements()) {                               // loop thru groups
            i++;
            Object obj = e.nextElement();
            String sGroup = ((JIDGroup)obj).getName();
            if ((sGroupName.equals(sGroup)))
                return i;
        }                               // end, loop thru groups
        return( -1 );
    }

    /**
     * Indicates that a group should be toggled.
     * @return true if successful, false otherwise.
     */
    public boolean toggleGroup() {
        Object o = getSelectedItem();
        if( o != null && o instanceof JIDGroup ) {
            ((JIDGroup)o).toggleExpanded();
            display();
            return true;
        }
        return false;
    }

    /**
     * Get JID
     * @param jid to get
     * @param nMask indicating how picky to be when getting jid
     * @return JID matching inputs + mask
     */
    public JID getJID(JID jid, int nMask) {
        // search all groups for the jid = input jid depending on the input mask.
        // for each group
        Enumeration e = vGroups.elements();
        JID fndJID;
        while (e.hasMoreElements()) {                                   // loop thru all groups in list
            Object obj = e.nextElement();
            fndJID = ((JIDGroup)obj).getJID(jid, nMask);
            if (fndJID != null)             // found jid in this group?
                return(fndJID);             // return the jid
        }                                   // end, loop thru all groups in list
        return(null);                       // matching jid not found
    }

    /**
     * Get JID if selected.
     * @return JID if selected.
     */
    public JID getSelectedJID() {
        if( isJIDSelected() )
            return (JID)getSelectedItem();
        return null;
    }


    /**
     * Get the element at the specified index. This is the model index.
     * @param index of element to retrieve
     * @return element at the specified index.
     */
    public Object getElementAt( int index ) {
        if( index >= 0 && index < list.size() )
            return list.elementAt(index);
        return null;
    }

    /**
     * Get the size of the list model's data.
     * @return the size of the list.
     */
    public int getSize() {
        return list.size();
    }

    /**
     * Get the selected index.
     */
    public int getSelectedIndex( ) {
        return selectedIndex;
    }
    /**
     * Set the selected index.
     * @param index of element to set
     */
    public void setSelectedIndex( int index ) {
        if( index < list.size() )
            selectedIndex = index;
    }

    /**
     * Set the selected item.
     * @param item to select
     */
    public void setSelectedItem( Object item ) {
        if( list.contains( item ) )
            selectedIndex = list.indexOf(item);
    }

    /**
     * Insert element into list
     * @param o object of element to insert
     * @param index of where to place object
     */
    public void insertElementAt( Object o, int index ) {
        list.insertElementAt( o, index );
        fireRosterChange( );
    }

    /**
     * add object to list
     * @param o object of element add
     */
    public void add
        ( Object o ) {
        list.addElement( o );
        fireRosterChange( );
    }

    /**
     * remove object from list
     * @param o object to remove
     */
    public boolean remove
        ( Object o ) {
        boolean ret = list.removeElement( o );
        fireRosterChange( );
        return ret;
    }

    /**
     * remove object from list
     * @param index of element to remove
     */
    public void removeElementAt( int index ) {
        list.removeElementAt( index );
        fireRosterChange( );
    }
}
