package org.jabber.applet.connection;

/***************************************************************************
 * The contents of this file are subject to the Jabber Open Source License
 * Version 1.0 (the "License").  You may not copy or use this file, in
 * either source code or executable form, except in compliance with the 
 * License.  You may obtain a copy of the License at 
 * 
 * http://www.jabber.com/license/ 
 * 
 * or at
 * 
 * http://www.opensource.org/.
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the
 * License for the specific language governing rights and limitations 
 * under the License.
 * 
 * Copyrights
 * 
 * Portions created by or assigned to Jabber.com, Inc. are
 * Copyright (c) 1999-2000 Jabber.com, Inc.  All Rights Reserved.  Contact
 * information for Jabber.com, Inc. is available at http://www.jabber.com/.
 * 
 * Acknowledgements
 * 
 * Special thanks to the Jabber Open Source Contributors for their
 * suggestions and support of Jabber.
 ***************************************************************************/

/**
 * Enumeration for Socket Connection State
 *
 * @author		Jonathan Paulson
 * @version 	$ 0.6.4 $ 
 */
public class ConnectionState {
    public static final int CLOSED				= 0;		//Default. CLOSED
    public static final int OPEN				= 1;		// OPEN
    public static final int LISTENING			= 2;		// LISTENING
    public static final int CONNECTION_PENDING	= 3;		// Connection pending
    public static final int CONNECTING			= 6;		// CONNECTING
    public static final int CONNECTED			= 7;		// CONNECTED
    public static final int CLOSING				= 8;		// Peer is closing the Connection
    public static final int ERROR				= -4000;	// ERROR
    private int             miState				= CLOSED;

    /**
     * Class Constructor
     *
     * @param newState		Value of the Connection state
     */
    public ConnectionState(int newState) {
        miState=newState;
    }

    /**
     * Class Constructor
     */
    public ConnectionState() {
        miState=CLOSED;
    }

    /**
     * Set Connection state
     *
     * @param newState		Value of the Connection state	 
     */
    public void state(int newState) {
        miState=newState;
        System.out.println("Setting connection state to: "+toString());
    }

    /**
     * Get Connection state
     *
     * @return	Connection state
     */
    public int state() {
        return miState;
    }

    /**
    * Returns the Connection State String Value
    * 
    * @return	Connection State
    */
    public String toString() {
        switch(miState) {
        case CLOSED:
            return "Closed";
        case OPEN:
            return "Open";
        case LISTENING:
            return "Listening";
        case CONNECTION_PENDING:
            return "Connection Pending";
        case CONNECTING:
            return "Connecting";
        case CONNECTED:
            return "Connected";
        case CLOSING:
            return "Closing";
        case ERROR:
            return "Error";
        default:
            return "UNKNOWN State";
        }
    }
}
